
#include "CVert.h"
#include "CTexCoord.h"
#include "common.h"
#include <string>

/*!
 * Spezifiert den Typ des Objektes
 */
enum CMeshObjectTyp{
	CRectangle,
	CCube
};

//! CMesh class
/*! 
 * Diese Klassse dient zur Erstellung der VBOs Objekte
 * @author Michael Beham
 * @date 4.12.2010
 */
class CMesh
{
public:
	// Mesh Data
	int		m_nVertexCount;						// Vertex Count
	CVert*	m_pVertices;						// Vertex Data
	CVert*	m_pColors;							// Color Data
	CTexCoord*	m_pTextures;							// Texture Coordinates

	// Vertex Buffer Object Names
	unsigned int	m_nVBOVertices;						// Vertex VBO Name
	unsigned int	m_nVBOColors;
	unsigned int	m_nVBOTextures;

public:
	/*!
	 * Konstruktor Erstellt die Arrays und setzt die Werte, welche fr die Erstellung der VBOs bentigt werden
	 * @param [in] type Type der Geometrie
	 */
	CMesh(CMeshObjectTyp type )								// Mesh Constructor
	{
		typ = type;
		switch(typ)
		{
		case CCube:
			m_nVertexCount = 24;
			m_pVertices = new CVert[m_nVertexCount];
			m_pColors = new CVert[m_nVertexCount];

			setVertex(0, 1.0f, 1.0f,0.0f);			// Top Right Of The Quad (Top)
			setVertex(1, 0.0f, 1.0f,0.0f);			// Top Left Of The Quad (Top)
			setVertex(2, 0.0f, 1.0f,1.0f);			// Bottom Left Of The Quad (Top)
			setVertex(3, 1.0f, 1.0f,1.0f);			// Bottom Right Of The Quad (Top)

			setVertex(4, 1.0f,0.0f, 1.0f);			// Top Right Of The Quad (Bottom)
			setVertex(5, 0.0f,0.0f, 1.0f);			// Top Left Of The Quad (Bottom)
			setVertex(6, 0.0f,0.0f, 0.0f);				// Bottom Left Of The Quad (Bottom)
			setVertex(7, 1.0f,0.0f, 0.0f);			// Bottom Right Of The Quad (Bottom)

			setVertex(8, 1.0f, 1.0f, 1.0f);			// Top Right Of The Quad (Front)
			setVertex(9, 0.0f, 1.0f, 1.0f);			// Top Left Of The Quad (Front)
			setVertex(10,0.0f,0.0f, 1.0f);			// Bottom Left Of The Quad (Front)
			setVertex(11,1.0f,0.0f, 1.0f);			// Bottom Right Of The Quad (Front)

			setVertex(12,1.0f,0.0f,0.0f);			// Bottom Left Of The Quad (Back)
			setVertex(13,0.0f,0.0f,0.0f);				// Bottom Right Of The Quad (Back)
			setVertex(14,0.0f, 1.0f,0.0f);			// Top Right Of The Quad (Back)
			setVertex(15,1.0f, 1.0f,0.0f);			// Top Left Of The Quad (Back)

			setVertex(16,0.0f, 1.0f, 1.0f);			// Top Right Of The Quad (Left)
			setVertex(17,0.0f, 1.0f,0.0f);			// Top Left Of The Quad (Left)
			setVertex(18,0.0f,0.0f,0.0f);				// Bottom Left Of The Quad (Left)
			setVertex(19,0.0f,0.0f, 1.0f);			// Bottom Right Of The Quad (Left)

			setVertex(20,1.0f, 1.0f,0.0f);			// Top Right Of The Quad (Right)
			setVertex(21,1.0f, 1.0f, 1.0f);			// Top Left Of The Quad (Right)
			setVertex(22,1.0f,0.0f, 1.0f);			// Bottom Left Of The Quad (Right)
			setVertex(23,1.0f,0.0f,0.0f);			// Bottom Right Of The Quad (Right)
		break;
		case CRectangle:
			m_nVertexCount = 4;
			m_pVertices = new CVert[m_nVertexCount];
			m_pTextures = new CTexCoord[m_nVertexCount];
			setVertex(0, 0.0, 0.0,-1,0.0f, 0.0f);
			setVertex(1, 1.0, 0.0,-1, 1.0f, 0.0f);
			setVertex(2, 1.0, 1.0,-1,1.0f, 1.0f);
			setVertex(3, 0.0, 1.0,-1,0.0f, 1.0f);
		break;
		}
	};
	/*!
	* Dekonstruktor
	*/
	~CMesh();								// Mesh Deconstructor

	/*!
	 * Setzt die Vertices
	 * @param i Index von Array
	 * @param x x Koordinate
	 * @param y y Koordinate
	 * @param z z Koordinate
	 */
	void setVertex(int i, float x,float y, float z)
	{
		m_pVertices[i].x =x;
		m_pVertices[i].y =y;
		m_pVertices[i].z =z;		
		m_pColors[i].x =x;
		m_pColors[i].y =y;
		m_pColors[i].z =z;
	};

	/*!
	 * Setzt die Vertices
	 * @param i Index von Array
	 * @param x x Koordinate
	 * @param y y Koordinate
	 * @param z z Koordinate
	 * @param z u Textur-Koordinate
	 * @param z v Textur-Koordinate
	 */
	void setVertex(int i, float x,float y, float z, float u, float v)
	{
		m_pVertices[i].x =x;
		m_pVertices[i].y =y;
		m_pVertices[i].z =z;		
		m_pTextures[i].u =u;
		m_pTextures[i].v =v;
	};

	/*!
	*Diese Methode erstellt die VBs. Anschlieend befindet sich die Arrays in der Grafikkarte und knnen gerendert werden.
	*/
	void BuildVBOs()
	{
		switch(typ)
		{
		case CCube:		
			glEnableClientState( GL_VERTEX_ARRAY );				// Disable Vertex Arrays
			glEnableClientState( GL_COLOR_ARRAY );				// Disable Color Arrays

			// Generate And Bind The Vertex Buffer
			glGenBuffersARB( 1, &m_nVBOVertices );							// Get A Valid Name
			glBindBufferARB( GL_ARRAY_BUFFER_ARB, m_nVBOVertices );			// Bind The Buffer
			// Load The Data
			glBufferDataARB( GL_ARRAY_BUFFER_ARB, m_nVertexCount*3*sizeof(float), m_pVertices, GL_STATIC_DRAW_ARB );

			// Generate And Bind The Texture Coordinate Buffer
			glGenBuffersARB( 1, &m_nVBOColors );					// Get A Valid Name
			glBindBufferARB( GL_ARRAY_BUFFER_ARB, m_nVBOColors );		// Bind The Buffer
			// Load The Data
			glBufferDataARB( GL_ARRAY_BUFFER_ARB, m_nVertexCount*3*sizeof(float), m_pColors, GL_STATIC_DRAW_ARB );

			// Our Copy Of The Data Is No Longer Necessary, It Is Safe In The Graphics Card
			//delete [] m_pVertices; m_pVertices = NULL;
			//delete [] m_pColors; m_pColors = NULL;

			std::cout << "VBOs Vertex " << m_nVBOVertices << std::endl;
			std::cout << "VBOs Color " << m_nVBOColors << std::endl;
			std::cout << "VBOs BUILDED" << std::endl;

			//glDisableClientState(GL_COLOR_ARRAY);
			//glDisableClientState(GL_VERTEX_ARRAY); 
			break;
		case CRectangle:
			glEnableClientState( GL_VERTEX_ARRAY );				// Disable Vertex Arrays
			glEnableClientState( GL_TEXTURE_COORD_ARRAY );				// Disable Color Arrays

			// Generate And Bind The Vertex Buffer
			glGenBuffersARB( 1, &m_nVBOVertices );							// Get A Valid Name
			glBindBufferARB( GL_ARRAY_BUFFER_ARB, m_nVBOVertices );			// Bind The Buffer
			// Load The Data
			glBufferDataARB( GL_ARRAY_BUFFER_ARB, m_nVertexCount*3*sizeof(float), m_pVertices, GL_STATIC_DRAW_ARB );

			glClientActiveTexture(GL_TEXTURE1);
			// Generate And Bind The Texture Coordinate Buffer
			glGenBuffersARB( 1, &m_nVBOTextures );					// Get A Valid Name
			glBindBufferARB( GL_ARRAY_BUFFER_ARB, m_nVBOTextures );		// Bind The Buffer
			// Load The Data
			glBufferDataARB( GL_ARRAY_BUFFER_ARB, m_nVertexCount*2*sizeof(float), m_pTextures, GL_STATIC_DRAW_ARB );

			// Our Copy Of The Data Is No Longer Necessary, It Is Safe In The Graphics Card
			//delete [] m_pVertices; m_pVertices = NULL;
			//delete [] m_pColors; m_pColors = NULL;

			std::cout << "VBOs Vertex " << m_nVBOVertices << std::endl;
			std::cout << "VBOs Color " << m_nVBOColors << std::endl;
			std::cout << "VBOs BUILDED" << std::endl;

			//glDisableClientState(GL_TEXTURE_COORD_ARRAY);
			//glDisableClientState(GL_VERTEX_ARRAY); 
			break;
		}
	};

	private:
		CMeshObjectTyp typ;
};
