#pragma once

#include "common.h"
//!  Color class. 
/*!
 * Diese Klasse dient zur Verwaltung der Farbwerte und war Teil des vorgegebenen Frameworks.
 * @author Unbekannt
*/

// a (r,g,b,a) color value
class Color
{
public:

	Color(const unsigned char uR=0, const unsigned char uG=0, const unsigned char uB=0, const unsigned char uA=255)
	{
		m_uData[0] = (float)uR;
		m_uData[1] = (float)uG;
		m_uData[2] = (float)uB;
		m_uData[3] = (float)uA;
	};

	Color(const int iR, const int iG, const int iB, const int iA=255)
	{
		m_uData[0] =  std::max(0,std::min(255,iR));
		m_uData[1] =  std::max(0,std::min(255,iG));
		m_uData[2] =  std::max(0,std::min(255,iB));
		m_uData[3] =  std::max(0,std::min(255,iA));
	};

	Color(const float fR, const float fG, const float fB, const float fA=1.0f)
	{
		m_uData[0] =  std::max(0.0f,std::min(255.0f,fR*255.0f));
		m_uData[1] =  std::max(0.0f,std::min(255.0f,fG*255.0f));
		m_uData[2] =  std::max(0.0f,std::min(255.0f,fB*255.0f));
		m_uData[3] =  std::max(0.0f,std::min(255.0f,fA*255.0f));
	};

	Color(const unsigned char *pData)
	{
		m_uData[0] = (float)pData[0];
		m_uData[1] = (float)pData[1];
		m_uData[2] = (float)pData[2];
		m_uData[3] = (float)pData[3];
	};

	Color(const float *pData)
	{
		m_uData[0] = std::max(0.0f,std::min(255.0f,pData[0]*255.0f));
		m_uData[1] = std::max(0.0f,std::min(255.0f,pData[1]*255.0f));
		m_uData[2] = std::max(0.0f,std::min(255.0f,pData[2]*255.0f));
		m_uData[3] = std::max(0.0f,std::min(255.0f,pData[3]*255.0f));
	};

	~Color()
	{
	};

	void Set(const unsigned char uR, const unsigned char uG, const unsigned char uB, const unsigned char uA = 255)
	{
		m_uData[0] = (float)uR;
		m_uData[1] = (float)uG;
		m_uData[2] = (float)uB;
		m_uData[3] = (float)uA;
	};

	const float * Get() const
	{
		return &m_uData[0];
	};

	void SetRed(const float uR)
	{
		m_uData[0] = (float) uR;
	};

	const float GetRed() const
	{
		return m_uData[0];
	};

	void SetGreen(const float uG)
	{
		m_uData[1] = uG;
	};

	const float GetGreen() const
	{
		return m_uData[1];
	};

	void SetBlue(const float uB)
	{
		m_uData[2] = uB;
	};

	const float GetBlue() const
	{
		return m_uData[2];
	};

	void SetAlpha(const float uA)
	{
		m_uData[3] = uA;
	};

	const float GetAlpha() const
	{
		return m_uData[3];
	};

	void SetNormalized(const float fR, const float fG, const float fB, const float fA = 1.0f)
	{
		m_uData[0] = (unsigned char) std::max(0.0f,std::min(255.0f,fR*255.0f));
		m_uData[1] = (unsigned char) std::max(0.0f,std::min(255.0f,fG*255.0f));
		m_uData[2] = (unsigned char) std::max(0.0f,std::min(255.0f,fB*255.0f));
		m_uData[3] = (unsigned char) std::max(0.0f,std::min(255.0f,fA*255.0f));
	};

	void SetNormalizedRed(const float fR)
	{
		m_uData[0] = (unsigned char) std::max(0.0f,std::min(255.0f,fR*255.0f));
	};

	void SetNormalizedGreen(const float fG)
	{
		m_uData[1] = (unsigned char) std::max(0.0f,std::min(255.0f,fG*255.0f));
	};

	void SetNormalizedBlue(const float fB)
	{
		m_uData[2] = (unsigned char) std::max(0.0f,std::min(255.0f,fB*255.0f));
	};

	void SetNormalizedAlpha(const float fA)
	{
		m_uData[3] = (unsigned char) std::max(0.0f,std::min(255.0f,fA*255.0f));
	};

	const float GetNormalizedRed() const
	{
		return float(m_uData[0]) / 255.0f;
	};

	const float GetNormalizedGreen() const
	{
		return float(m_uData[1]) / 255.0f;
	};

	const float GetNormalizedBlue() const
	{
		return float(m_uData[2]) / 255.0f;
	};

	const float GetNormalizedAlpha() const
	{
		return float(m_uData[3]) / 255.0f;
	};

	void SetHSV(const float fHue, const float fSaturation, const float fValue)
	{
		float fRed = fValue;
		float fGreen = fValue;
		float fBlue = fValue;
		
		if(fSaturation > 0.0f)
		{
			const float fH = fHue * 6.0f;
			const unsigned int i = unsigned int(fH);
			const float fF = fH - i;

			const float fP = fValue * (1.0f - fSaturation);
			const float fQ = fValue * (1.0f - (fSaturation * fF));
			const float fT = fValue * (1.0f - (fSaturation * (1.0f - fF)));

			switch(i)
			{
			case 0:			
				fGreen = fT;
				fBlue = fP;
				break;
			case 1:
				fRed = fQ;
				fBlue = fP;
				break;
			case 2:
				fRed = fP;
				fBlue = fT;
				break;
			case 3:
				fRed = fP;
				fGreen = fQ;
				break;
			case 4:
				fRed = fT;
				fGreen = fP;
				break;
			case 5:
				fGreen = fP;
				fBlue = fQ;
				break;
			}
		}

		SetNormalized(fRed,fGreen,fBlue,GetNormalizedAlpha());
	};

	const Color & operator*=(const float fOther)
	{
		m_uData[0] = std::max(0.0f,std::min(255.0f,float(m_uData[0]) * fOther));
		m_uData[1] = std::max(0.0f,std::min(255.0f,float(m_uData[1]) * fOther));
		m_uData[2] = std::max(0.0f,std::min(255.0f,float(m_uData[2]) * fOther));
		m_uData[3] = std::max(0.0f,std::min(255.0f,float(m_uData[3]) * fOther));
		return *this;
	};

	const Color & operator/=(const float fOther)
	{
		m_uData[0] = std::max(0.0f,std::min(255.0f,float(m_uData[0]) / fOther));
		m_uData[1] = std::max(0.0f,std::min(255.0f,float(m_uData[1]) / fOther));
		m_uData[2] = std::max(0.0f,std::min(255.0f,float(m_uData[2]) / fOther));
		m_uData[3] = std::max(0.0f,std::min(255.0f,float(m_uData[3]) / fOther));
		return *this;
	};

	const Color & operator+=(const Color &colOther)
	{
		m_uData[0] = std::min(255,int(m_uData[0])+int(colOther.m_uData[0]));
		m_uData[1] = std::min(255,int(m_uData[1])+int(colOther.m_uData[1]));
		m_uData[2] = std::min(255,int(m_uData[2])+int(colOther.m_uData[2]));
		m_uData[3] = std::min(255,int(m_uData[3])+int(colOther.m_uData[3]));
		return *this;
	};

	const Color & operator-=(const Color &colOther)
	{
		m_uData[0] =  std::max(0,int(m_uData[0])-int(colOther.m_uData[0]));
		m_uData[1] =  std::max(0,int(m_uData[1])-int(colOther.m_uData[1]));
		m_uData[2] =  std::max(0,int(m_uData[2])-int(colOther.m_uData[2]));
		m_uData[3] =  std::max(0,int(m_uData[2])-int(colOther.m_uData[3]));
		return *this;
	};

	const Color operator*(const float fOther) const
	{
		Color colNew = *this;
		colNew *= fOther;
		return colNew;
	};

	const Color operator/(const float fOther) const
	{
		Color colNew = *this;
		colNew *= fOther;
		return colNew;
	};

	const Color operator+(const Color &colOther) const
	{
		Color colNew = *this;
		colNew += colOther;
		return colNew;
	};

	const Color operator-(const Color &colOther) const
	{
		Color colNew = *this;
		colNew -= colOther;
		return colNew;
	};

	const bool operator==(const Color & colOther) const
	{
		return (memcmp((void*)m_uData,(void*)colOther.m_uData,sizeof(float)*4) == 0);
	};

	const bool operator!=(const Color & colOther) const
	{
		return !(*this == colOther);
	};

	const bool choose()
	{
		return chooseColor(*this);
	};

	static const bool chooseColor(Color & colColor)
	{
		CHOOSECOLOR cc;
		static COLORREF vcCustom[16];
		
		ZeroMemory(&cc, sizeof(cc));
		cc.lStructSize = sizeof(cc);
		cc.hwndOwner = GetForegroundWindow();
		cc.lpCustColors = (LPDWORD) vcCustom;
		cc.rgbResult = RGB(colColor.GetRed(),colColor.GetGreen(),colColor.GetBlue());
		cc.Flags = CC_ANYCOLOR | CC_FULLOPEN | CC_RGBINIT;

		const bool bResult = (ChooseColor(&cc) != 0);

		if (bResult)
		{
			colColor.SetRed(GetRValue(cc.rgbResult));
			colColor.SetGreen(GetGValue(cc.rgbResult));
			colColor.SetBlue(GetBValue(cc.rgbResult));
		}

		return bResult;
	}

private:
	float m_uData[4];
};

inline std::ostream & operator<< (std::ostream & os, const Color & colColor)
{
	os << "(" << unsigned int(colColor.GetRed()) << ";" << unsigned int(colColor.GetGreen()) << ";" << unsigned int(colColor.GetBlue()) << ";" << unsigned short(colColor.GetAlpha()) << ")";
	return os;
}

inline std::istream & operator>> (std::istream & is, Color & colColor)
{	
	unsigned int uR,uG,uB,uA;

	if (is >> eat("(") >> uR >> eat(";") >> uG >> eat(";") >> uB >> eat(";") >> uA >> eat(")"))
		colColor.Set(float(uR),float(uG),float(uB),float(uA));

	return is;
}
