#pragma once
#include "common.h"
#include "Vector.h"

//!  Font class. 
/*!
 * Diese Klasse dient zur Verwaltung der darzustellenden Wrter (z.B. Perpective, X-View,...). 
 * Im Endeffekt beinhaltet diese Klasse das Wort, welches dargestellt werden soll, sowie Methoden zum Zeichnen.
 * @author Michael Beham
 * @date 4.11.2010 
*/

class Font
{
public:
	/**
	* Standardkonstruktor: Setzt Text auf "", sowie die Position auf 0,0,-1
	**/
	Font() : m_pos(0,0,-1)
	{
		m_text = "";
	};

	/**
	* Konstruktor: Setzt Text auf dem Inhalt von der bergebenen Variable, sowie die Position auf 0,0,-1.
	* @param[in] text Der darzustellende Text
	**/
	Font(std::string text) : m_pos(0,0,-1)
	{
		m_text = text;
	};

	/**
	* SetText: Setzt Text auf dem Inhalt von der bergebenen Variable.
	* @param[in] text Der darzustellende Text
	**/
	void SetText(std::string text)
	{
		m_text = text;
	};

	/**
	* GetText: Liefert den darzustellenden Text.
	* @return Der darzustellende Text
	**/
	std::string GetText()
	{
		return m_text;
	};

	/**
	* SetPosition: Setzt die Position des darzustellenden Text.
	* @param[in] pos Position des darzustellenden Textes.
	**/
	void SetPosition(Vector pos)
	{
		m_pos = pos;
	};

	/**
	* GetPosition: Liefert die Position des darzustellenden Textes.
	* @return Position des darzustellenden Textes.
	**/
	Vector GetPosition()
	{
		return m_pos;
	};

	/**
	* Draw: Zeichnet den Text an einer gewnschten Position.
	* @param[in] x X Koordinate der gewnschten Position
	* @param[in] y Y Koordinate der gewnschten Position
	* @param[in] z Z Koordinate der gewnschten Position
	**/
	void Draw(float x, float y, float z)
	{
		const unsigned char* vpMessage = (const unsigned char*) m_text.c_str();
		glColor4f(1.0f,1.0f,1.0f,1.0f);
		glRasterPos3f(x, y, z);
		glutBitmapString(GLUT_BITMAP_HELVETICA_12,vpMessage);
		glRasterPos2f(0.0f,0.0f);	
	};

	/**
	* Draw: Zeichnet den Text.
	**/
	void Draw()
	{
		const unsigned char* vpMessage = (const unsigned char*) m_text.c_str();
		glColor4f(1.0f,1.0f,1.0f,1.0f);
		glRasterPos3f(m_pos.GetX(), m_pos.GetY(), m_pos.GetZ());
		//glRasterPos3f(0, 0, m_pos.GetZ());
		glutBitmapString(GLUT_BITMAP_HELVETICA_12,vpMessage);
		glRasterPos2f(0.0f,0.0f);	
	};

private:
	std::string m_text;
	Vector m_pos;
};