#ifndef LOAD_SOUND_H
#define LOAD_SOUND_H

#include "LoadData.h"
#include "TFContainer.h"


//! LoadTramsferFunction class
/*!
 * Diese Klasse ladet die Transferfunktion, indem sie die Werte aus einer XML Funktion rausliest.
 * Die Klasse erbt die Funktionen von LoadData.
*/
class LoadTramsferFunction : public LoadData
{
public:
	/**
	 * Konstruktor: Ladet die Datei, welche zum bergebenen Path verknpft wurde.
	 * @param [in] file Pfad zur Datei, welche geladen werden soll.
	 */
	LoadTramsferFunction(const char* file);
	/**
	 * Dekonstruktor
	 */
	~LoadTramsferFunction();

	/**
	 * Liefert alle Punkte, welche geladen wurden.
	 * @return Punkte der Transferfunktion.
	 */
	vector<TFContainer> GetPoints();	
	
protected:
	/**
	 * Fr jedes Tag in der XML wird diese Funktion aufgerufen.
	 * @param [in] pParent Parent Objekt
	 * @param [in] indent Ebene des Tags
	 */
	void loadingTag(TiXmlNode* pParent, unsigned int indent);
	/**
	* Fr jedes Attribut in der XML Datei wird diese Funktion aufgerufen
	* @param [in] pParent Parent Objekt
	* @param [in] indent Ebene des Tags
	* @return 0
	*/
	int attribsLoading(TiXmlElement* pElement, unsigned int indent);

private:
	//variables for internal use.
	bool failedLoading;
	std::vector<TFContainer> points;
};

#endif