#include "common.h"
#include "AbstractShader.h"

//!  PostProcessingShaderObj class. 
/*!
 * Diese Klasse dient als Container der Postprocessing-Shader. Einerseits wird die ShaderID gespeichert, anderseits die IDs fr die Objekte der Shader.
 * Alle Shader werden gleich behandelt, sprich besitzen alle die gleichen Uniform Attribute und Vertex Shader. Dadurch lsst sich die Komplexitt der Shaderverwaltung reduzieren.
 * (Stichwort: Alles ber die Leiste scherren)
 * @author Michael Beham
 * @date 4.12.2010
*/
class PostProcessingShaderObj
{
public:
	GLuint shaderID;
	GLuint Texture;
	GLuint lightPos;
	GLuint weight;
	GLuint exposure;
	GLuint density;
	GLuint decay;
};

//!PostProcessingShader class
/*!
 * Diese Klasse dient zur Verwaltung der Shader fr das Postprocessing. 
 * Sie erbt die Funktionen von AbstractShader und erstellt, startet, stopt und lscht die Shader, welche mit Hilfe der Funktionen von Abstract Shader erstellt wurden.
 * @author Michael Beham
 * @date 4.12.2010
*/
class PostProcessingShader : public AbstractShader
{
public:
	/*!
	 * Konstruktor: Erstellt alle Shader fr das Postprocessing
	 */
	PostProcessingShader()
	{
		gTransformQuadric = createShader("ls.vert","ls.frag");
	};

	/*!
	 * Lscht alle Shaderobjekte, welche erstellt wurden
	 */
	~PostProcessingShader()
	{
		glDeleteShader(gTransformQuadric.shaderID);
	};

	/*!
	 * Diese Funktion erstellt den jeweiligen Shader und verlinkt die ShaderID, sowie dessen Attribute.
	 * @param [in] vertexshadername Pfad zum Vertexshader
	 * @param [in] fragmentshadername Pfad zum Fragmentshader
	 * @return Verlinktes Shaderobjekt
	 */
	PostProcessingShaderObj createShader(const char* vertexshadername, const char* fragmentshadername)
	{
		PostProcessingShaderObj shaderID;
		GLhandleARB vertexShaderHandle;
		GLhandleARB fragmentShaderHandle;
		vertexShaderHandle   = loadShader(vertexshadername,GL_VERTEX_SHADER);
		fragmentShaderHandle = loadShader(fragmentshadername,GL_FRAGMENT_SHADER);
		shaderID.shaderID = glCreateProgramObjectARB();
		glAttachObjectARB(shaderID.shaderID,vertexShaderHandle);
		glAttachObjectARB(shaderID.shaderID,fragmentShaderHandle);
		glLinkProgramARB(shaderID.shaderID);

		shaderID.Texture = glGetUniformLocationARB(shaderID.shaderID,"Texture");
		shaderID.lightPos = glGetUniformLocationARB(shaderID.shaderID,"lightPos");
		shaderID.weight = glGetUniformLocationARB(shaderID.shaderID,"weight");
		shaderID.exposure = glGetUniformLocationARB(shaderID.shaderID,"exposure");
		shaderID.density = glGetUniformLocationARB(shaderID.shaderID,"density");
		shaderID.decay = glGetUniformLocationARB(shaderID.shaderID, "decay");

		return shaderID;
	}

	/*
	void start(Vector lightPos, float weight, float exposure, float decay,float density )
	{
		PostProcessingShaderObj Shader = gTransformQuadric;
		
		glUseProgramObjectARB(Shader.shaderID);
		glUniform1iARB(Shader.Texture, 0);
		glUniform2fARB(Shader.lightPos, lightPos.GetX(), lightPos.GetY());
		glUniform1fARB(Shader.weight,weight);
		glUniform1fARB(Shader.exposure,exposure);
		glUniform1fARB(Shader.density ,density );
		glUniform1fARB(Shader.decay,decay);
	};

	void stop()
	{
		glUseProgramObjectARB(0);
	};*/

private:
	PostProcessingShaderObj gTransformQuadric;
};
