#include "AbstractShader.h"

//!  ShaderObj class. 
/*!
 * Diese Klasse dient als Container der Shader. Einerseits wird die ShaderID gespeichert, anderseits die IDs fr die Objekte der Shader.
 * Alle Shader werden gleich behandelt, sprich besitzen alle die gleichen Uniform Attribute und Vertex Shader. Dadurch lsst sich die Komplexitt der Shaderverwaltung reduzieren.
 * (Stichwort: Alles ber die Leiste scherren)
 * @author Michael Beham
 * @date 4.12.2010
*/
class ShaderObj
{
public:
	GLuint shaderID;
	GLuint sbackface;
	GLuint stransfer_func;
	GLuint spartiell;
	GLuint sLightSourcePos;
	GLuint IsMode;
	GLuint sSlideValue;
	GLuint scentral_differences;
	GLuint sMaxIntensity;
	GLuint sPreshading;
	//GLuint sCartoon;
	GLuint sInterpolation;
	GLuint sStepsize;
	GLuint sResolution;
	GLuint sAdaptiv;
	GLuint sBackground;
	GLuint sClipping;
	GLuint sTransferGradient;
};

//!Shader class
/*!
 * Diese Klasse dient zur Verwaltung der Shader fr das Raycasting. 
 * Sie erbt die Funktionen von AbstractShader und erstellt, startet, stopt und lscht die Shader, welche mit Hilfe der Funktionen von Abstract Shader erstellt wurden.
 * @author Michael Beham
 * @date 4.12.2010
*/
class Shader : public AbstractShader
{
public:
	/*!
	 * Konstruktor: Erstellt alle Shader fr das Raycasting
	 */
	Shader()
	{
		gTransformQuadric = createShader("gquadric_vert.glsl","gquadric_frag.glsl");
		gSlices = createShader("gquadric_vert.glsl","visu_slices.frag");
		gVisu = createShader("gquadric_vert.glsl","visu.frag");
		gVisuAvg = createShader("gquadric_vert.glsl","visu_avg.frag");
		gVisuFirst = createShader("gquadric_vert.glsl","visu_first.frag");
		gVisuMIP = createShader("gquadric_vert.glsl","visu_mip.frag");
		gVisuShadow = createShader("gquadric_vert.glsl","visu_shadow.frag");
		gVisuAvgShadow = createShader("gquadric_vert.glsl","visu_avg_shadow.frag");
		gVisuFirstShadow = createShader("gquadric_vert.glsl","visu_first_shadow.frag");
		gVisuMIPShadow = createShader("gquadric_vert.glsl","visu_mip_shadow.frag");
		gVisuContour = createShader("gquadric_vert.glsl","visu_contour.frag");
		gVisuAvgContour = createShader("gquadric_vert.glsl","visu_avg_contour.frag");
		gVisuFirstContour = createShader("gquadric_vert.glsl","visu_first_contour.frag");
		gVisuMIPContour = createShader("gquadric_vert.glsl","visu_mip_contour.frag");

		gVisuExploration = createShader("gquadric_vert.glsl","visu_exploration.frag");
		gVisuAvgExploration = createShader("gquadric_vert.glsl","visu_avg_exploration.frag");
		gVisuFirstExploration = createShader("gquadric_vert.glsl","visu_first_exploration.frag");
		gVisuMIPExploration = createShader("gquadric_vert.glsl","visu_mip_exploration.frag");

		//glDeleteShader(vertexShaderHandle);
		//glDeleteShader(fragmentShaderHandle);
	};

	/*!
	 * Lscht alle Shaderobjekte, welche erstellt wurden
	 */
	~Shader()
	{
		glDeleteShader(gTransformQuadric.shaderID);
		glDeleteShader(gSlices.shaderID);
		glDeleteShader(gVisu.shaderID);
		glDeleteShader(gVisuAvg.shaderID);
		glDeleteShader(gVisuFirst.shaderID);
		glDeleteShader(gVisuMIP.shaderID);
		glDeleteShader(gVisuShadow.shaderID);
		glDeleteShader(gVisuAvgShadow.shaderID);
		glDeleteShader(gVisuFirstShadow.shaderID);
		glDeleteShader(gVisuMIPShadow.shaderID);
		glDeleteShader(gVisuContour.shaderID);
		glDeleteShader(gVisuAvgContour.shaderID);
		glDeleteShader(gVisuFirstContour.shaderID);
		glDeleteShader(gVisuMIPContour.shaderID);
		glDeleteShader(gVisuExploration.shaderID);
		glDeleteShader(gVisuAvgExploration.shaderID);
		glDeleteShader(gVisuFirstExploration.shaderID);
		glDeleteShader(gVisuMIPExploration.shaderID);
	}

	/*!
	 * Diese Funktion erstellt den jeweiligen Shader und verlinkt die ShaderID, sowie dessen Attribute.
	 * @param [in] vertexshadername Pfad zum Vertexshader
	 * @param [in] fragmentshadername Pfad zum Fragmentshader
	 * @return Verlinktes Shaderobjekt
	 */
	ShaderObj createShader(const char* vertexshadername, const char* fragmentshadername)
	{
		ShaderObj shaderID;
		GLhandleARB vertexShaderHandle;
		GLhandleARB fragmentShaderHandle;
		vertexShaderHandle   = loadShader(vertexshadername,GL_VERTEX_SHADER);
		fragmentShaderHandle = loadShader(fragmentshadername,GL_FRAGMENT_SHADER);
		shaderID.shaderID = glCreateProgramObjectARB();
		glAttachObjectARB(shaderID.shaderID,vertexShaderHandle);
		glAttachObjectARB(shaderID.shaderID,fragmentShaderHandle);
		glLinkProgramARB(shaderID.shaderID);

		shaderID.sbackface = glGetUniformLocationARB(shaderID.shaderID,"sBackFace");
		shaderID.stransfer_func = glGetUniformLocationARB(shaderID.shaderID,"sTransfer");
		shaderID.spartiell = glGetUniformLocationARB(shaderID.shaderID,"sPartiel");
		shaderID.sLightSourcePos = glGetUniformLocationARB(shaderID.shaderID,"sLightSourcePos");
		shaderID.IsMode = glGetUniformLocationARB(shaderID.shaderID,"IsMode");
		shaderID.sSlideValue = glGetUniformLocationARB(shaderID.shaderID, "sSlideValue");
		shaderID.scentral_differences = glGetUniformLocationARB(shaderID.shaderID, "preprocess");
		shaderID.sMaxIntensity = glGetUniformLocationARB(shaderID.shaderID, "sMaxIntensity");
		shaderID.sPreshading = glGetUniformLocationARB(shaderID.shaderID, "sPreshading");
		//shaderID.sCartoon = glGetUniformLocationARB(shaderID.shaderID, "sCartoon");
		shaderID.sInterpolation = glGetUniformLocationARB(shaderID.shaderID, "sInterpolation");
		shaderID.sResolution = glGetUniformLocationARB(shaderID.shaderID, "resolution");
		shaderID.sStepsize = glGetUniformLocationARB(shaderID.shaderID, "stepsize");
		shaderID.sAdaptiv = glGetUniformLocationARB(shaderID.shaderID, "sAdaptiv");
		shaderID.sBackground = glGetUniformLocationARB(shaderID.shaderID, "sBackground");
		shaderID.sClipping = glGetUniformLocationARB(shaderID.shaderID, "sClipping");
		shaderID.sTransferGradient = glGetUniformLocationARB(shaderID.shaderID, "sTransferGradient");

	
		return shaderID;
	}
	
	/*!
	 * Diese Funktion startet den Shader.
	 * @param [in] shadingmode Welche Renderingvariante soll bentzt werden (Contour, Standard, Shadow, Exploration?)
	 * @param [in] slidingx X Koordinate der Lichtquelle
	 * @param [in] slidingy Y Koordinate der Lichtquelle
	 * @param [in] slidingz Z Koordinate der Lichtquelle
	 * @param [in] slidemode Soll nur Slices gezeichnet werden.
	 * @param [in] slicing Bestimmt darzustellenden Slice
	 * @param [in] slidevalue Bestimmt darzustellenden Slice
	 * @param [in] central_differences Sollen die Gradienten mithilfe der Central Differences berechnet werden.
	 * @param [in] maxintensity Wie sollen die Ray abgetastet werden (MIP, Standard, First, Avg)
	 * @param [in] preshading Soll das Objekt geshadet werden (mithilfe von Phong)
	 * @param [in] interpolation Sollen die Werte interpoliert werden
	 * @param [in] resolution Auflsung des FBOs
	 * @param [in] steps Anzahl der Schritte
	 * @param [in] adaptiv Soll adaptiv gesamplt werden
	 */
	void start(int shadingmode, float slidingx, float slidingy, float slidingz, int slidemode, int slicing, float slidevalue,int central_differences, int maxintensity, int preshading, int interpolation, Vector resolution, float steps, int adaptiv,int clipping, int withgradient)
	{
		ShaderObj Shader = gTransformQuadric;

		if(slicing == 1 && slidemode > 1) Shader = gSlices;
		else
		{
		switch(shadingmode)
		{
		case 1:
			switch(maxintensity)
			{
			case 0: Shader = gVisuContour;	
				break;
			case 1: Shader = gVisuMIPContour;
				break;
			case 2: Shader = gVisuAvgContour;
				break;
			case 3: Shader = gVisuFirstContour;
				break;
			}
			break;
		case 2: 
			switch(maxintensity)
			{
			case 0: Shader = gVisuShadow;	
				break;
			case 1: Shader = gVisuMIPShadow;
				break;
			case 2: Shader = gVisuAvgShadow;
				break;
			case 3: Shader = gVisuFirstShadow;
				break;
			}
			break;
		case 3:
			switch(maxintensity)
			{
				case 0: Shader = gVisuExploration;	
						break;
				case 1: Shader = gVisuMIPExploration;
					break;
				case 2: Shader = gVisuAvgExploration;
					break;
				case 3: Shader = gVisuFirstExploration;
					break;
			}
			break;
		case 0:
			switch(maxintensity)
			{
				case 0: Shader = gVisu;	
					break;
				case 1: Shader = gVisuMIP;
					break;
				case 2: Shader = gVisuAvg;
					break;
				case 3: Shader = gVisuFirst;
					break;
				}	
			}
		}
		
		if(slicing == 0)
		{
			glUseProgramObjectARB(Shader.shaderID);
			glUniform1iARB(Shader.sBackground, 0);
			glUniform1iARB(Shader.sbackface, 1);
			glUniform1iARB(Shader.stransfer_func,3);
			glUniform1iARB(Shader.spartiell,4);
			glUniform3fARB(Shader.sLightSourcePos, slidingx,slidingy,slidingz);
			glUniform1iARB(Shader.IsMode, 1);
			glUniform1iARB(Shader.scentral_differences,central_differences);
			//glUniform1iARB(Shader.sCartoon,5);
			glUniform1iARB(Shader.sInterpolation,interpolation);
			glUniform3fARB(Shader.sResolution,1.0f/resolution.GetX(), 1.0f/resolution.GetY(), 1.0f/resolution.GetZ());
			glUniform1fARB(Shader.sStepsize,1.0f/steps);
			glUniform1iARB(Shader.sAdaptiv,adaptiv);
			glUniform1iARB(Shader.sClipping,clipping);
			glUniform1iARB(Shader.sTransferGradient, withgradient);
		}
		else
		{
			glUseProgramObjectARB(Shader.shaderID);
			glUniform1iARB(Shader.sBackground, 0);
			glUniform1iARB(Shader.sbackface, 1);
			glUniform1iARB(Shader.stransfer_func,3);
			glUniform1iARB(Shader.spartiell,4);
			glUniform3fARB(Shader.sLightSourcePos, slidingx,slidingy,slidingz);
			glUniform1iARB(Shader.IsMode, slidemode);
			glUniform1fARB(Shader.sSlideValue, slidevalue);
			glUniform1iARB(Shader.scentral_differences,central_differences);
			glUniform1iARB(Shader.sMaxIntensity, maxintensity);
			glUniform1iARB(Shader.sPreshading, preshading); 
			//glUniform1iARB(Shader.sCartoon,5);
			glUniform1iARB(Shader.sInterpolation,interpolation);
			glUniform3fARB(Shader.sResolution,1.0f/resolution.GetX(), 1.0f/resolution.GetY(), 1.0f/resolution.GetZ());
			glUniform1fARB(Shader.sStepsize,1.0f/steps);
			glUniform1iARB(Shader.sAdaptiv,adaptiv);
			glUniform1iARB(Shader.sClipping,clipping);
			glUniform1iARB(Shader.sTransferGradient, withgradient);
		}
	};

	/*!
	* Diese Funktion stoppt den Shader
	*/
	void stop()
	{
		glUseProgramObjectARB(0);
	};

private:
	ShaderObj gTransformQuadric;
	ShaderObj gSlices;
	ShaderObj gVisu;	
	ShaderObj gVisuAvg;
	ShaderObj gVisuFirst;
	ShaderObj gVisuMIP;
	ShaderObj gVisuShadow;
	ShaderObj gVisuAvgShadow;
	ShaderObj gVisuFirstShadow;
	ShaderObj gVisuMIPShadow;
	ShaderObj gVisuContour;
	ShaderObj gVisuAvgContour;
	ShaderObj gVisuFirstContour;
	ShaderObj gVisuMIPContour;
	ShaderObj gVisuExploration;
	ShaderObj gVisuAvgExploration;
	ShaderObj gVisuFirstExploration;
	ShaderObj gVisuMIPExploration;
};
