#pragma once

#include "Vector.h"

//! Transferfunction Container class
/*!
 * Die KLasse TFContainer ist eine Container Klasse fr die Transferfunktion.
 * Jedes Objekt dieser Klasse entspricht einem vom Benutzer erstellten Punkt des GUI Elements der Transferfunktion.
 * Mathematisch gesehen ist jedes Objekt ein Sttzpunkt der Transferfunktion. 
 * <p>
 * Ein Sttzpunkt bentigt  einerseits eine Farbe, anderseis die Koordinate. Die Farbe wird in ein Float Array gespeichert, die Koordinate in der Parentklasse Vector.
 * </p>
 * @author: Michael Beham
 * @date: 4.12.2010
*/
class TFContainer : public Vector
{
public:
	/*!
	* Standardkonstruktor: Erstellt Sttzpunkt auf Koordinate 0,0 mit Farbe (0,0,0)
	*/
	TFContainer() : Vector()
	{
		m_vColor[0] = 0.0f;
		m_vColor[1] = 0.0f;
		m_vColor[2] = 0.0f;
	};

	/*!
	* Konstruktor: Erstellt den Sttzpunkt anhand der bergebenen Farbe und Koordinate
	* @param [in] fX X-Koordiante des Sttzpunktes
	* @param [in] fY Y-Koordiante des Sttzpunktes
	* @param [in] fR Rotanteil der Farbe des Sttzpunktes
	* @param [in] fG Grnanteil der Farbe des Sttzpunktes
	* @param [in] fB Blauanteil der Farbe des Sttzpunktes
	*/
	TFContainer(const float fX, const float fY, const float fR, const float fG, const float fB): Vector(fX,fY,0)
	{
		m_vColor[0] = fR;
		m_vColor[1] = fG;
		m_vColor[2]= fB;
	};

	/*!
	* Konstruktor: Erstellt den Sttzpunkt anhand der bergebenen Farbe und Koordinate
	* @param [in] vfValues Koordianten des Sttzpunktes
	* @param [in] cfValues Farbe des Sttzpunktes
	*/
	TFContainer::TFContainer(const float vfValues[], const float cfValues[]) : Vector(vfValues)
	{
		m_vColor[0] = cfValues[0];
		m_vColor[1] = cfValues[1];
		m_vColor[2] = cfValues[2];
	};

	/*!
	* Dekonstruktor
	*/
	~TFContainer()
	{
	};

	/*!
	* Diese Fkt. liefert den Rotanteil der Farbe.
	* @return Rotanteil der Farbe.
	*/
	const float GetR() const
	{
		return m_vColor[0];
	};

	/*!
	* Diese Fkt. liefert den Grnanteil der Farbe.
	* @return Grnuanteil der Farbe.
	*/
	const float GetG() const
	{
		return m_vColor[1];
	};

	/*!
	* Diese Fkt. liefert den Blauanteil der Farbe.
	* @return Blauanteil der Farbe.
	*/
	const float GetB() const
	{
		return m_vColor[2];
	};

	/*!
	* Diese Fkt. setzt den Rotanteil der Farbe.
	* @param [in] fR Neuer Rotanteil der Farbe.
	*/
	void SetR(const float fR)
	{
		m_vColor[0] = fR;
	};

	/*!
	* Diese Fkt. setzt den Grnanteil der Farbe.
	* @param [in] fG Neuer Grnanteil der Farbe.
	*/
	void SetG(const float fG)
	{
		m_vColor[1] = fG;
	};

	/*!
	* Diese Fkt. setzt den Blauanteil der Farbe.
	* @param [in] fB Neuer Blauanteil der Farbe.
	*/
	void SetB(const float fB)
	{
		m_vColor[2] = fB;
	};

	/*!
	* berladung des Operators < zur Sortierung der Punkte.
	* @param [in] i Vector, welcher die Koordinaten beinhaltet mit dem das Objekt verglichen werden soll.
	*/
	const bool operator<(Vector i) const
	{ 
		return GetX()<i.GetX();
	};

private:
	float m_vColor[3];
};